# -*- coding: utf-8 -*-
from PIL import Image
from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.forms import modelformset_factory
from django.forms.models import inlineformset_factory
from django.http import Http404

from billetterie.utils.forms import ModelFormWithFrozenField
from guichet.models import (
    PaypalAccount,
    Event,
    UserManager,
    GroupManager,
    Ticket,
    Option,
    Info_Sup,
    Info_Sup_Choice,
    Info_Sup_Choice_Choices,
    Info_Sup_Int,
    Info_Sup_Float,
    Info_Sup_File,
    Info_Sup_Recv,
    LydiaAccount,
)
from guichet.utils.registration import MIN_TEMPLATE_SIZE
from guichet.widgets import (
    LabelWithDescription,
    DescriptedCheckboxSelectMultiple,
    DateTimePicker,
    CustomFieldFile,
)

__all__ = [
    "PaypalAccountForm",
    "EventForm",
    "OpenTicketForm",
    "RestrictedTicketForm",
    "GodsonForm",
    "RegistrationForm",
    "UserManagerForm",
    "GroupManagerForm",
    "CheckForm",
    "TermOfUseForm",
    "ChoiceOptionsForm",
    "ChoiceOptionsWithGroupForm",
    "InfoSupCreationForm",
    "InfoSupChoicesSetCreationFormSet",
    "InfoSupChoicesSetEditFormSet",
    "InfoSupReceivedForm",
    "OptionAloneCreationForm",
    "OptionGroupCreationForm",
    "OptionGroupSetCreationFormSet",
    "OptionGroupSetEditFormSet",
    "LydiaAccountForm",
    "LydiaPaymentForm",
    "LydiaTokenForm",
]


class EventForm(forms.ModelForm):
    def __init__(self, *argv, **kwargs):
        instance = kwargs.get("instance", None)
        super(EventForm, self).__init__(*argv, **kwargs)
        if instance is None:
            self.fields["conditions_field"] = forms.BooleanField(
                required=True, label="J'accepte la charte de la billetterie en tant qu'organisateur."
            )

    paypal_account = forms.ModelChoiceField(
        queryset=PaypalAccount.objects,
        empty_label=None,
        required=True,
        label="Email du compte PayPal associé à l’évènement",
    )

    lydia_account = forms.ModelChoiceField(
        queryset=LydiaAccount.objects, empty_label=None, required=True, label="Compte Lydia associé à l’évènement"
    )

    class Meta:
        model = Event
        fields = [
            "name",
            "start_date",
            "end_date",
            "description",
            "email_event",
            "email_message",
            "other_payment_description",
            "terms_of_use",
            "limit_godson",
            "limit_ticket",
            "paypal_account",
            "lydia_account",
            "event_confidentiality",
        ]
        widgets = {
            "start_date": DateTimePicker(options=False),
            "end_date": DateTimePicker(options=False),
        }

    def is_valid(self):
        valid = super(EventForm, self).is_valid()

        if not valid:
            return valid

        if self.cleaned_data["end_date"] < self.cleaned_data["start_date"]:
            self.add_error(
                "end_date", ValidationError("La date de fin doit être après le début de l'évènement.", code="invalid")
            )
            valid = False

        return valid


class PaypalAccountForm(forms.ModelForm):
    class Meta:
        model = PaypalAccount
        fields = ["email", "description"]


class LydiaAccountForm(forms.ModelForm):
    class Meta:
        model = LydiaAccount
        fields = ["vendor_token", "private_token", "description"]


class LydiaTokenForm(forms.Form):
    phone_regex = RegexValidator(regex=r"^\+?1?\d{9,15}$", message="Numéro invalide.")
    phone_number = forms.CharField(max_length=17, label="Numéro de téléphone", validators=[phone_regex])
    password = forms.CharField(widget=forms.PasswordInput(), label="Mot de passe")


class LydiaPaymentForm(forms.Form):
    phone_regex = RegexValidator(regex=r"^\+?1?\d{9,15}$", message="Numéro invalide.")
    phone_number = forms.CharField(max_length=17, label="Numéro de téléphone", validators=[phone_regex])


class OpenTicketForm(ModelFormWithFrozenField):
    class Meta:
        model = Ticket
        fields = [
            "name",
            "short_notes",
            "long_notes",
            "price",
            "online_payment",
            "limit",
            "min_purchase_date",
            "max_purchase_date",
            "design",
            "admin_ticket",
            "allow_anonyme",
        ]
        widgets = {
            "max_purchase_date": DateTimePicker(options=False),
            "min_purchase_date": DateTimePicker(options=False),
            "design": CustomFieldFile(),
        }

    def is_valid(self):
        valid = super(OpenTicketForm, self).is_valid()

        if not valid:
            return valid

        if (
            self.cleaned_data["max_purchase_date"] is not None
            and self.cleaned_data["min_purchase_date"] is not None
            and self.cleaned_data["min_purchase_date"] > self.cleaned_data["max_purchase_date"]
        ):
            self.add_error(
                "min_purchase_date",
                ValidationError("La date limite de vente doit être après la date de début de vente", code="invalid"),
            )
            valid = False

        # Assure que le template a une taille suffisante
        if self.cleaned_data["design"]:
            with Image.open(self.cleaned_data["design"]) as template:
                width, height = template.size
                if width < MIN_TEMPLATE_SIZE.width or height < MIN_TEMPLATE_SIZE.height:
                    self.add_error(
                        "design",
                        ValidationError(
                            f"""La taille de l'image est trop petite (taille \
                                                   minimale requise: {MIN_TEMPLATE_SIZE.width} \
                                                   x{MIN_TEMPLATE_SIZE.height}px)""",
                            code="invalid",
                        ),
                    )
                    valid = False

        return valid


class RestrictedTicketForm(ModelFormWithFrozenField):
    class Meta:
        model = Ticket
        fields = [
            "name",
            "short_notes",
            "long_notes",
            "price",
            "online_payment",
            "limit",
            "min_purchase_date",
            "max_purchase_date",
            "design",
            "year_choice",
            "required_school",
            "required_ae",
            "groups",
            "allowed_godson",
            "admin_ticket",
            "allow_anonyme",
        ]
        widgets = {
            "max_purchase_date": DateTimePicker(options=False),
            "min_purchase_date": DateTimePicker(options=False),
            "design": CustomFieldFile(),
        }

    def is_valid(self):
        valid = super(RestrictedTicketForm, self).is_valid()

        if not valid:
            return valid

        if (
            self.cleaned_data["max_purchase_date"] is not None
            and self.cleaned_data["min_purchase_date"] is not None
            and self.cleaned_data["min_purchase_date"] > self.cleaned_data["max_purchase_date"]
        ):
            self.add_error(
                "min_purchase_date",
                ValidationError("La date limite de vente doit être après la date de début de vente", code="invalid"),
            )
            valid = False

        # Assure que le template a une taille suffisante
        if self.cleaned_data["design"]:
            with Image.open(self.cleaned_data["design"]) as template:
                width, height = template.size
                if width < MIN_TEMPLATE_SIZE.width or height < MIN_TEMPLATE_SIZE.height:
                    self.add_error(
                        "design",
                        ValidationError(
                            f"""La taille de l'image est trop petite (taille \
                                                   minimale requise: {MIN_TEMPLATE_SIZE.width} \
                                                   x{MIN_TEMPLATE_SIZE.height}px)""",
                            code="invalid",
                        ),
                    )
                    valid = False

        return valid


class OptionAloneCreationForm(ModelFormWithFrozenField):
    class Meta:
        model = Option
        fields = ["name", "price", "description", "admin_only"]


class OptionGroupCreationForm(forms.Form):

    group_name = forms.CharField(max_length=64, label="Nom du groupe d’option")
    description = forms.CharField(widget=forms.Textarea, label="Description de l’option", required=False)
    mandatory = forms.BooleanField(
        required=False, label="Option obligatoire (l'une des options doit être gratuite et disponible)"
    )


class OptionGroupSetCreationForm(ModelFormWithFrozenField):
    class Meta:
        model = Option
        fields = ["name", "price", "admin_only"]


OptionGroupSetCreationFormSet = modelformset_factory(
    Option, form=OptionGroupSetCreationForm, extra=0, min_num=1, can_delete=True
)
OptionGroupSetEditFormSet = modelformset_factory(Option, form=OptionGroupSetCreationForm, extra=0)


class InfoSupCreationForm(ModelFormWithFrozenField):
    class Meta:
        model = Info_Sup
        fields = ["name", "description", "after_payement"]

    @staticmethod
    def get_info_form(info):
        if info not in [i for i, __ in Info_Sup.INFO_TYPE]:
            raise Http404
        associated_form = {
            Info_Sup.INFO_SIMPLE: InfoSupCreationForm,
            Info_Sup.INFO_SHORT_TEXT: InfoSupCreationFormText,
            Info_Sup.INFO_LONG_TEXT: InfoSupCreationFormText,
            Info_Sup.INFO_CHOICE: InfoSupCreationFormChoice,
            Info_Sup.INFO_INT: InfoSupCreationFormInt,
            Info_Sup.INFO_FLOAT: InfoSupCreationFormFloat,
            Info_Sup.INFO_FILE: InfoSupCreationFormFile,
        }
        return associated_form[info]

    @staticmethod
    def get_frozen_field(info):
        if info not in [i for i, __ in Info_Sup.INFO_TYPE]:
            raise Http404
        frozen = ["after_payement"]
        associated_dic = {
            Info_Sup.INFO_SIMPLE: [],
            Info_Sup.INFO_SHORT_TEXT: [],
            Info_Sup.INFO_LONG_TEXT: [],
            Info_Sup.INFO_CHOICE: [],
            Info_Sup.INFO_INT: ["min_value", "max_value"],
            Info_Sup.INFO_FLOAT: ["min_value", "max_value"],
            Info_Sup.INFO_FILE: ["file_name"],
        }
        return frozen + associated_dic[info]


class FormInfoSupEditable(ModelFormWithFrozenField):
    def is_valid(self):
        valid = super(FormInfoSupEditable, self).is_valid()

        if not valid:
            return valid

        if self.cleaned_data["info_editable"] and not self.cleaned_data["info_validate"]:
            self.add_error(
                "info_editable",
                ValidationError(
                    "Cette option ne peux être activée que sans l'option d'auto-validation", code="invalid"
                ),
            )
            valid = False

        return valid


class InfoSupCreationFormText(FormInfoSupEditable):
    class Meta:
        model = Info_Sup
        fields = ["name", "description", "after_payement", "info_validate", "info_editable"]


class InfoSupCreationFormChoice(FormInfoSupEditable):
    class Meta:
        model = Info_Sup_Choice
        fields = ["name", "description", "after_payement", "info_validate", "info_editable"]


InfoSupChoicesSetCreationFormSet = inlineformset_factory(
    Info_Sup_Choice, Info_Sup_Choice_Choices, fields=("name",), extra=0, min_num=1, can_delete=True
)
InfoSupChoicesSetEditFormSet = inlineformset_factory(
    Info_Sup_Choice, Info_Sup_Choice_Choices, fields=("name",), extra=0
)


class FormInfoSupValue(FormInfoSupEditable):
    def is_valid(self):
        valid = super(FormInfoSupValue, self).is_valid()

        if not valid:
            return valid

        if (
            self.cleaned_data["min_value"] is not None
            and self.cleaned_data["max_value"] is not None
            and self.cleaned_data["min_value"] > self.cleaned_data["max_value"]
        ):
            self.add_error(
                "min_value",
                ValidationError("La valeur minimale doit être inférieure à la valeur maximale", code="invalid"),
            )
            valid = False

        return valid


class InfoSupCreationFormInt(FormInfoSupValue):
    class Meta:
        model = Info_Sup_Int
        fields = ["name", "description", "after_payement", "info_validate", "info_editable", "min_value", "max_value"]


class InfoSupCreationFormFloat(FormInfoSupValue):
    class Meta:
        model = Info_Sup_Float
        fields = ["name", "description", "after_payement", "info_validate", "info_editable", "min_value", "max_value"]


class InfoSupCreationFormFile(FormInfoSupEditable):
    class Meta:
        model = Info_Sup_File
        fields = ["name", "description", "after_payement", "info_validate", "info_editable", "file_name"]


class InfoSupReceivedForm(object):
    def __init__(self, *args, **kwargs):
        registration = kwargs.pop("registration")
        self.l_forms = []
        for info in registration.get_list_info_sup():
            self.l_forms.append(
                (info, self.get_info_received_form(info.reference.type_info)(*args, info_sup=info, **kwargs))
            )

    def get_forms(self):
        return self.l_forms

    def is_valid(self):
        return all(map(lambda x: x[1].is_valid(), self.l_forms))

    def uuid_check(self):
        return all(map(lambda x: x[1].uuid_check(), self.l_forms))

    def get_updated_objects(self):
        updated = []
        for info, form in self.l_forms:
            new_info = form.get_updated_object()
            if new_info.version != Info_Sup_Recv.objects.get(pk=info.pk).version:
                updated.append(new_info)
        return updated

    @staticmethod
    def get_info_received_form(info):
        if info not in [i for i, __ in Info_Sup.INFO_TYPE]:
            raise Http404
        associated_form = {
            Info_Sup.INFO_SIMPLE: InfoSupCommonReceivedForm,
            Info_Sup.INFO_SHORT_TEXT: InfoSupShortTextReceivedForm,
            Info_Sup.INFO_LONG_TEXT: InfoSupLongTextReceivedForm,
            Info_Sup.INFO_CHOICE: InfoSupChoicesReceivedForm,
            Info_Sup.INFO_INT: InfoSupIntReceivedForm,
            Info_Sup.INFO_FLOAT: InfoSupFloatReceivedForm,
            Info_Sup.INFO_FILE: InfoSupFileReceivedForm,
        }
        return associated_form[info]


class InfoSupCommonReceivedForm(forms.Form):

    ADMIN_CHOICE = (
        (Info_Sup_Recv.STATE_ACCEPTED, "Accepter l'information"),
        (Info_Sup_Recv.STATE_PENDING, "Information en attente de validation"),
        (Info_Sup_Recv.STATE_REJECTED, "Rejeter l'information"),
        (Info_Sup_Recv.STATE_MISSING, "Aucune information reçue"),
    )

    def __init__(self, *args, **kwargs):
        self.info_sup = kwargs.pop("info_sup")
        self.admin_form = kwargs.pop("admin_form", False)
        super(InfoSupCommonReceivedForm, self).__init__(*args, **kwargs)
        self.prefixe_field_name = "info-{}".format(self.info_sup.pk)
        self.fields[self.prefixe_field_name + "-uuid"] = forms.UUIDField(
            widget=forms.HiddenInput(), initial=self.info_sup.version
        )
        if self.admin_form:
            if self.info_sup.state != Info_Sup_Recv.STATE_ACCEPTED:
                self.fields[self.prefixe_field_name + "-state"] = forms.ChoiceField(
                    choices=self.ADMIN_CHOICE, required=True, initial=self.info_sup.state
                )
            else:
                self.fields[self.prefixe_field_name + "-state"] = forms.ChoiceField(
                    choices=self.ADMIN_CHOICE, required=False, disabled=True, initial=self.info_sup.state
                )
                self.fields[self.prefixe_field_name + "-state"].widget.attrs["readonly"] = True
                self.fields[self.prefixe_field_name + "-state"].widget.attrs["disabled"] = True
        self.value_required = (
            self.info_sup.state == Info_Sup_Recv.STATE_ACCEPTED
            and self.info_sup.reference.info_editable
            and not self.admin_form
        )

    def uuid_check(self):
        # if uuid has changed:
        #   1. a raced condition occure between the get (actual value of uuid field)
        #        and the post (initial value of uuid field)
        #   2. user change the hidden field. It's his problem.
        return not self.fields[self.prefixe_field_name + "-uuid"].has_changed(
            initial=self.info_sup.version, data=self.cleaned_data[self.prefixe_field_name + "-uuid"]
        )

    def get_updated_object(self):
        info = self.info_sup
        if self.admin_form and info.state != Info_Sup_Recv.STATE_ACCEPTED:
            if self.info_sup.state != int(self.cleaned_data[self.prefixe_field_name + "-state"]):
                info.state = int(self.cleaned_data[self.prefixe_field_name + "-state"])
                info.update_version()
        return info


class InfoSupCommonValueReceivedForm(InfoSupCommonReceivedForm):
    def is_valid(self):
        valid = super(InfoSupCommonValueReceivedForm, self).is_valid()

        if not valid:
            return valid

        if (
            self.info_sup.state == Info_Sup_Recv.STATE_ACCEPTED
            and self.info_sup.reference.info_editable
            and not self.admin_form
            and not self.cleaned_data[self.prefixe_field_name + "-value"]
        ):
            self.add_error(
                self.prefixe_field_name + "-value", ValidationError("Une valeur doit être présente.", code="invalid")
            )
            valid = False

        return valid

    def get_updated_object(self):
        info = super(InfoSupCommonValueReceivedForm, self).get_updated_object()
        if (
            self.admin_form
            or self.info_sup.state != Info_Sup_Recv.STATE_ACCEPTED
            or self.info_sup.reference.info_editable
        ) and self.fields[self.prefixe_field_name + "-value"].has_changed(
            initial=self.info_sup.value, data=self.cleaned_data[self.prefixe_field_name + "-value"]
        ):
            info.value = self.cleaned_data[self.prefixe_field_name + "-value"]
            if self.admin_form:
                pass
            elif self.info_sup.reference.info_validate:
                info.state = Info_Sup_Recv.STATE_ACCEPTED
            elif not self.cleaned_data[self.prefixe_field_name + "-value"]:
                info.state = Info_Sup_Recv.STATE_MISSING
            elif info.state != Info_Sup_Recv.STATE_ACCEPTED and self.cleaned_data[self.prefixe_field_name + "-value"]:
                info.state = Info_Sup_Recv.STATE_PENDING
        return info

    def _froze_value(self):
        if (
            self.info_sup.state == Info_Sup_Recv.STATE_ACCEPTED
            and not self.admin_form
            and not self.info_sup.reference.info_editable
        ):
            self.fields[self.prefixe_field_name + "-value"].disabled = True
            self.fields[self.prefixe_field_name + "-value"].widget.attrs["readonly"] = True
            self.fields[self.prefixe_field_name + "-value"].widget.attrs["disabled"] = True


class InfoSupShortTextReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupShortTextReceivedForm, self).__init__(*args, **kwargs)
        self.fields[self.prefixe_field_name + "-value"] = forms.CharField(
            max_length=128,
            strip=True,
            label=self.info_sup.reference.name,
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
        )
        self._froze_value()


class InfoSupLongTextReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupLongTextReceivedForm, self).__init__(*args, **kwargs)
        self.fields[self.prefixe_field_name + "-value"] = forms.CharField(
            strip=True,
            label=self.info_sup.reference.name,
            widget=forms.widgets.Textarea(),
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
        )
        self._froze_value()


class InfoSupChoicesReceivedField(forms.ModelChoiceField):
    def label_from_instance(self, obj):
        return obj.name


class InfoSupChoicesReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupChoicesReceivedForm, self).__init__(*args, **kwargs)
        self.fields[self.prefixe_field_name + "-value"] = InfoSupChoicesReceivedField(
            queryset=self.info_sup.reference.get_sup_type().choices.all(),
            label=self.info_sup.reference.name,
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
        )
        self._froze_value()


class InfoSupIntReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupIntReceivedForm, self).__init__(*args, **kwargs)
        param = self.info_sup.reference.get_sup_type()
        self.fields[self.prefixe_field_name + "-value"] = forms.IntegerField(
            label=self.info_sup.reference.name,
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
            min_value=param.min_value,
            max_value=param.max_value,
        )
        self._froze_value()


class InfoSupFloatReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupFloatReceivedForm, self).__init__(*args, **kwargs)
        param = self.info_sup.reference.get_sup_type()
        self.fields[self.prefixe_field_name + "-value"] = forms.FloatField(
            label=self.info_sup.reference.name,
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
            min_value=param.min_value,
            max_value=param.max_value,
        )
        self._froze_value()


class CustomFileField(forms.FileField):
    def has_changed(self, initial, data):
        if not super(CustomFileField, self).has_changed(initial, data):
            return False
        return initial != data


class InfoSupFileReceivedForm(InfoSupCommonValueReceivedForm):
    def __init__(self, *args, **kwargs):
        super(InfoSupFileReceivedForm, self).__init__(*args, **kwargs)
        self.fields[self.prefixe_field_name + "-value"] = CustomFileField(
            label=self.info_sup.reference.name,
            initial=self.info_sup.value,
            required=self.value_required,
            validators=self.info_sup.reference.get_validators(),
            widget=CustomFieldFile(),
        )
        self._froze_value()


class ChoiceOptionsForm(forms.Form):
    def __init__(self, *args, **kwargs):
        options = kwargs.pop("options")
        super(ChoiceOptionsForm, self).__init__(*args, **kwargs)
        ol = []
        for o in options:
            ol.append((o.id, LabelWithDescription(o.name + " (+ " + str(o.price) + "€)", o.description)))
        self.fields["options"] = forms.MultipleChoiceField(
            required=False, widget=DescriptedCheckboxSelectMultiple, choices=ol
        )


class ChoiceOptionsWithGroupForm(forms.Form):
    def __init__(self, *args, **kwargs):
        queryset = kwargs.pop("queryset").order_by("price", "name")
        index = kwargs.pop("index")
        super(ChoiceOptionsWithGroupForm, self).__init__(*args, **kwargs)
        if queryset.first().mandatory:
            self.fields["choice-{}".format(index)] = forms.ModelChoiceField(
                queryset=queryset, empty_label=None, required=True
            )
        else:
            self.fields["choice-{}".format(index)] = forms.ModelChoiceField(
                queryset=queryset, empty_label="Sans l’option", required=False
            )
        self.fields["choice-{}".format(index)].label = LabelWithDescription(
            queryset.first().group_name, queryset.first().description
        )


# Acheter une place pour un ami / famille


class GodsonForm(forms.Form):

    payee = forms.CharField(max_length=64, label="Prénom et nom du bénéficiaire")
    conditions_field = forms.BooleanField(required=True)


# Réserver une place pour un autre compte (admin only)
class AccountForm(forms.Form):

    email = forms.EmailField(max_length=128)


class TermOfUseForm(forms.Form):

    conditions_field = forms.BooleanField(required=True)


class RegistrationForm(forms.Form):

    login = forms.CharField(
        max_length=128,
        required=False,
        label="À partir d’un login INP-net",
        widget=forms.TextInput(attrs={"placeholder": "Login INP-net"}),
    )
    email = forms.CharField(
        max_length=128,
        required=False,
        label="À partir d’une adresse email",
        widget=forms.TextInput(attrs={"placeholder": "Adresse email"}),
    )
    anonyme = forms.BooleanField(required=False, label="Vente sur place (Sans compte sur la billetterie)")
    payee = forms.CharField(
        max_length=64,
        required=False,
        label="Bénéficiaire (uniquement si parrainage ou vente sur place)",
        widget=forms.TextInput(attrs={"placeholder": "Bénéficiaire"}),
    )
    force_condition = forms.BooleanField(
        required=False, label="Forcer les restrictions du billet (promo, école, club, parrainage)"
    )
    conditions_field = forms.BooleanField(required=True)


class UserManagerForm(forms.ModelForm):

    login = forms.CharField(
        max_length=128,
        required=False,
        label="À partir d’un login INP-net",
        widget=forms.TextInput(attrs={"placeholder": "Login INP-net"}),
    )
    email = forms.CharField(
        max_length=128,
        required=False,
        label="À partir d’une adresse email",
        widget=forms.TextInput(attrs={"placeholder": "Adresse email"}),
    )

    class Meta:
        model = UserManager
        fields = ["access"]


class GroupManagerForm(forms.ModelForm):
    class Meta:
        model = GroupManager
        fields = ["access", "group", "bureau_only"]


class CheckForm(forms.Form):

    search = forms.CharField(
        max_length=32, label="Billet ou carte AE", widget=forms.TextInput(attrs={"autofocus": True})
    )
